#include "AT91SAM7S256.h"
#include "Board.h"
#include "flashprog.h"

#define AT91C_MC_CORRECT_KEY  ((unsigned int) 0x5A << 24) // (MC) Correct Protect Key

//*----------------------------------------------------------------------------
// AT91F_Flash_Write
// Funkcja zapisu danych do Flasha / zapisuje caa stron (256 bajtow dla SAM7S256)
// Flash_Address: start at 0x0010 0000 size: in byte
// definicje dla pamieci FLASH (znajduja sie w bibliotece AT91SAM7S256.h)
// AT91C_IFLASH	 ((char *) 	0x00100000)    // adres poczatkowy pamieci FLASH
// AT91C_IFLASH_SIZE	 ((unsigned int) 0x00040000) // rozmiar pamieci flash w bajtach
// AT91C_IFLASH_PAGE_SIZE	 ((unsigned int) 256)    // rozmiar strony w bajtach
// AT91C_IFLASH_NB_OF_PAGES	 ((unsigned int) 1024)   // ilosc stron
//*----------------------------------------------------------------------------
__attribute__ ((section (".ramfunc"))) int AT91F_Flash_Write( unsigned int Flash_Address ,int size ,unsigned int * buff)
{
    // adres bazowy rejestrow zwiazanych z obsluga FLASH
    AT91PS_MC ptMC = AT91C_BASE_MC;
    unsigned int i, page, status;
    unsigned int * Flash;

    status = 0;
    // Oczekiwanie na mozliwosc zapisu
	while ((status & AT91C_MC_FRDY) != AT91C_MC_FRDY )
	{
		status = AT91C_BASE_MC->MC_FSR;
	}
    // inicjalizacja wskaznika dla adresu pamieci pod ktory zostana zapisane dane
    Flash = (unsigned int *) Flash_Address;
    // Obliczenie strony pamieci na podstawie podanego adresu,calkowitego rozmiaru pamieci
    // i rozmiaru strony
    page = ((Flash_Address - (unsigned int)AT91C_IFLASH ) /FLASH_PAGE_SIZE);

    // kopiowanie wartosci w miejsce gdzie pamieci zostana zapisane dane
    for (i=0; (i < FLASH_PAGE_SIZE) & (size > 0) ;i++, Flash++,buff++,size-=4 ){
    // kopiujemy  po 4 bajty (bo zmienna Flash jest wskaznikiem na int)
    *Flash=*buff;
    }

    // Pobranie rejestru statusowego przerwan i ich wylaczenie
    unsigned long ulIntrStat = AT91C_BASE_AIC->AIC_IMR;
    AT91C_BASE_AIC->AIC_IDCR = 0xFFFFFFFF;

    // Komenda zapisu strony pamieci
    ptMC->MC_FCR =AT91C_MC_CORRECT_KEY | AT91C_MC_FCMD_START_PROG | (AT91C_MC_PAGEN & (page<<8));

    //Prowizoryczny "delay" blokujacy dalsze operacje na czas zapisu do FLASH
    for(i=0;i<100000;i++) asm("nop");

    status = 0;
    // Oczekiwanie na zakonczenie zapisu
     while ((status & AT91C_MC_FRDY) != AT91C_MC_FRDY )
     {
         status = AT91C_BASE_MC->MC_FSR;
     }

    //Wlaczenie przerwan
    AT91C_BASE_AIC->AIC_IECR = ulIntrStat;


    // jesli zapis sie nie powiedzie zostanie zwrocony "false"
    if ( (status & ( AT91C_MC_PROGE | AT91C_MC_LOCKE ))!=0)
    return false;
    return true;
}


// Odczyt danych
//*----------------------------------------------------------------------------
//* \fn    AT91F_Flash_Read
//* \brief Odczyt danych z Flash'a
//*----------------------------------------------------------------------------
void AT91F_Flash_Read(unsigned int add, unsigned int size, unsigned int * read_buff)
{
   unsigned int i;
   unsigned int * start_add = (unsigned int *) add;

    for (i=0 ; i < size ; i++)
    {
        *(read_buff + i) = *(start_add + i);
    }
}



